@@ -30,14 +30,6 @@ GST_DEBUG_CATEGORY_STATIC (gstmpegdemux_debug);
 #define PARSE_CLASS(o)  GST_MPEG_PARSE_CLASS (G_OBJECT_GET_CLASS (o))
 #define CLASS(o)  GST_MPEG_DEMUX_CLASS (G_OBJECT_GET_CLASS (o))
 
-/* elementfactory information */
-static GstElementDetails mpeg_demux_details = {
-  "MPEG Demuxer",
-  "Codec/Demuxer",
-  "Demultiplexes MPEG1 and MPEG2 System Streams",
-  "Erik Walthinsen <omega@cse.ogi.edu>\n" "Wim Taymans <wim.taymans@chello.be>"
-};
-
 /* MPEG2Demux signals and args */
 enum
 {
@@ -90,8 +82,6 @@ GST_STATIC_PAD_TEMPLATE ("private_%d",
 GST_BOILERPLATE_FULL (GstMPEGDemux, gst_mpeg_demux, GstMPEGParse,
     GST_TYPE_MPEG_PARSE, _do_init);
 
-static void gst_mpeg_demux_class_init (GstMPEGDemuxClass * klass);
-
 static gboolean gst_mpeg_demux_process_event (GstMPEGParse * mpeg_parse,
     GstEvent * event);
 
@@ -169,7 +159,10 @@ gst_mpeg_demux_base_init (gpointer klass_ptr)
   gst_element_class_add_pad_template (element_class, klass->audio_template);
   gst_element_class_add_pad_template (element_class, klass->private_template);
 
-  gst_element_class_set_details (element_class, &mpeg_demux_details);
+  gst_element_class_set_details_simple (element_class, "MPEG Demuxer",
+      "Codec/Demuxer",
+      "Demultiplexes MPEG1 and MPEG2 System Streams",
+      "Erik Walthinsen <omega@cse.ogi.edu>, Wim Taymans <wim.taymans@chello.be>");
 }
 
 static void
@@ -207,8 +200,8 @@ gst_mpeg_demux_class_init (GstMPEGDemuxClass * klass)
   klass->sync_stream_to_time = gst_mpeg_demux_sync_stream_to_time;
 
   /* we have our own sink pad template, but don't use it in subclasses */
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &sink_template);
 }
 
 static void
@@ -331,6 +324,7 @@ gst_mpeg_demux_init_stream (GstMPEGDemux * mpeg_demux,
   str->last_flow = GST_FLOW_OK;
   str->buffers_sent = 0;
   str->tags = NULL;
+  str->caps = NULL;
 }
 
 static GstMPEGStream *
@@ -373,6 +367,8 @@ gst_mpeg_demux_get_video_stream (GstMPEGDemux * mpeg_demux,
     GstTagList *list;
 
     /* We need to set new caps for this pad. */
+    if (str->caps)
+      gst_caps_unref (str->caps);
     str->caps = gst_caps_new_simple ("video/mpeg",
         "mpegversion", G_TYPE_INT, mpeg_version,
         "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
@@ -445,6 +441,8 @@ gst_mpeg_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
     GstTagList *list;
 
     /* We need to set new caps for this pad. */
+    if (str->caps)
+      gst_caps_unref (str->caps);
     str->caps = gst_caps_new_simple ("audio/mpeg",
         "mpegversion", G_TYPE_INT, 1, NULL);
     if (!gst_pad_set_caps (str->pad, str->caps)) {
@@ -499,11 +497,9 @@ static gboolean
 gst_mpeg_demux_parse_packhead (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
 {
   GstMPEGDemux *demux = GST_MPEG_DEMUX (mpeg_parse);
-  guint8 *buf;
 
   parent_class->parse_packhead (mpeg_parse, buffer);
 
-  buf = GST_BUFFER_DATA (buffer);
   /* do something useful here */
 
   if (demux->pending_tags) {
@@ -640,20 +636,19 @@ gst_mpeg_demux_parse_packet (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
   guint16 headerlen;
 
   guint16 packet_length;
-  gboolean STD_buffer_bound_scale;
-  guint16 STD_buffer_size_bound;
+  guint16 STD_buffer_size_bound G_GNUC_UNUSED;
   guint64 dts;
   gint64 pts = -1;
 
   guint16 datalen;
 
   GstMPEGStream *outstream = NULL;
-  guint8 *buf, *basebuf;
+  guint8 *buf;
   gint64 timestamp;
 
   GstFlowReturn ret = GST_FLOW_OK;
 
-  basebuf = buf = GST_BUFFER_DATA (buffer);
+  buf = GST_BUFFER_DATA (buffer);
   id = *(buf + 3);
   buf += 4;
 
@@ -681,7 +676,7 @@ gst_mpeg_demux_parse_packet (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
       case 0x40:
         GST_DEBUG_OBJECT (mpeg_demux, "have STD");
 
-        STD_buffer_bound_scale = bits & 0x20;
+        /* STD_buffer_bound_scale = ((bits & 0x20) == 0x20); */
         STD_buffer_size_bound = ((guint16) (bits & 0x1F)) << 8;
         STD_buffer_size_bound |= *buf++;
 
@@ -958,7 +953,7 @@ gst_mpeg_demux_combine_flows (GstMPEGDemux * demux, GstMPEGStream * stream,
   stream->last_flow = flow;
 
   /* if it's success we can return the value right away */
-  if (GST_FLOW_IS_SUCCESS (flow))
+  if (flow == GST_FLOW_OK)
     goto done;
 
   /* any other error that is not-linked can be returned right
@@ -1034,9 +1029,11 @@ gst_mpeg_demux_send_subbuffer (GstMPEGDemux * mpeg_demux,
       outstream->scr_offs = 0;
 
     if (mpeg_demux->index != NULL) {
-      /* Register a new index position. */
+      /* Register a new index position.
+       * FIXME: check for keyframes
+       */
       gst_index_add_association (mpeg_demux->index,
-          outstream->index_id, 0,
+          outstream->index_id, GST_ASSOCIATION_FLAG_DELTA_UNIT,
           GST_FORMAT_BYTES,
           GST_BUFFER_OFFSET (buffer), GST_FORMAT_TIME, timestamp, 0);
     }
@@ -1067,7 +1064,6 @@ gst_mpeg_demux_send_subbuffer (GstMPEGDemux * mpeg_demux,
     GstClockTimeDiff diff;
     guint64 update_time;
 
-    update_time = MIN (timestamp, mpeg_parse->current_segment.stop);
     update_time = MAX (timestamp, mpeg_parse->current_segment.start);
     diff = GST_CLOCK_DIFF (mpeg_parse->current_segment.last_stop, update_time);
     if (diff > GST_SECOND * 2) {
@@ -1334,6 +1330,8 @@ gst_mpeg_demux_reset (GstMPEGDemux * mpeg_demux)
           gst_event_new_eos ());
       gst_element_remove_pad (GST_ELEMENT (mpeg_demux),
           mpeg_demux->video_stream[i]->pad);
+      if (mpeg_demux->video_stream[i]->caps)
+        gst_caps_unref (mpeg_demux->video_stream[i]->caps);
       g_free (mpeg_demux->video_stream[i]);
       mpeg_demux->video_stream[i] = NULL;
     }
@@ -1345,6 +1343,8 @@ gst_mpeg_demux_reset (GstMPEGDemux * mpeg_demux)
           mpeg_demux->audio_stream[i]->pad);
       if (mpeg_demux->audio_stream[i]->tags)
         gst_tag_list_free (mpeg_demux->audio_stream[i]->tags);
+      if (mpeg_demux->audio_stream[i]->caps)
+        gst_caps_unref (mpeg_demux->audio_stream[i]->caps);
       g_free (mpeg_demux->audio_stream[i]);
       mpeg_demux->audio_stream[i] = NULL;
     }
@@ -1354,6 +1354,8 @@ gst_mpeg_demux_reset (GstMPEGDemux * mpeg_demux)
           gst_event_new_eos ());
       gst_element_remove_pad (GST_ELEMENT (mpeg_demux),
           mpeg_demux->private_stream[i]->pad);
+      if (mpeg_demux->private_stream[i]->caps)
+        gst_caps_unref (mpeg_demux->private_stream[i]->caps);
       g_free (mpeg_demux->private_stream[i]);
       mpeg_demux->private_stream[i] = NULL;
     }
